#define QUERY_INTERVAL 600000L  // 600초(10분) 간격으로 날씨 정보 요청
#define SERVER_TIMEOUT 10000L   // 서버 응답 대기 시간 10초

#define STATE_IDLE    0
#define STATE_RECEIVING_INFO  1

String buffer = "";     // 날씨 정보 데이터 저장 버퍼
char *connectString = "AT+CIPSTART=\"TCP\",\"api.openweathermap.org\",80";
char *infoString = "AT+CIPSEND=133";
char *dataString1 =
  "GET /data/2.5/weather?id=1838524&APPID=your_API_key_here HTTP/1.1";
char *dataString2 = "Host: api.openweathermap.org";
char *dataString3 = "Connection: close";

int state = STATE_IDLE;     // 프로그램의 상태

unsigned long time_previous, time_current;

void setup() {
  Serial.begin(115200);     // 컴퓨터와의 UART 시리얼 연결
  Serial1.begin(115200);      // ESP-01 모듈과의 UART 시리얼 연결

  Serial.println("** Ready to start...");
  delay(1000);

  // ESP-01 모듈 리셋
  if (!sendATcommand("AT+RST", "ready", 3)) {
    Serial.println("** Error during reset ESP-01...");
    while (1);
  }
  else {
    Serial.println("** ESP-01 has started...");
  }

  // 소프트AP+스테이션 모드로 설정
  if (!sendATcommand("AT+CWMODE=3", "OK", 3)) {
    Serial.println("** Error in changing mode...");
    while (1);
  }
  else {
    Serial.println("** ESP-01 has been set to mode 3...");
  }

  // AP 접속 (ssid와 password를 접속하고자 하는 AP의 정보로 변경하여야 함)
  if (!sendATcommand("AT+CWJAP=\"ssid\",\"password\"", "OK", 10)) {
    Serial.println("** Error in connecting to AP...");
    while (1);
  }
  else {
    Serial.println("** ESP-01 has been connected to AP...");
  }

  time_current = millis();
  time_previous = time_current - QUERY_INTERVAL;
}

boolean sendATcommand(char *command, char *response, int second) {
  // 최대 대기 시간 설정
  Serial1.setTimeout(second * 1000);
  // AT 명령을 ESP-01 모듈로 전달하여 실행
  Serial1.println(command);
  // 응답 대기
  boolean responseFound = Serial1.find(response);
  // 최대 대기 시간을 디폴트값인 1초로 변경
  Serial1.setTimeout(1000);

  return responseFound;     // AT 명령 실행 성공 여부 반환
}

void loop() {
  time_current = millis();

  // 이전 날씨 정보 요청 이후 일정 시간이 경과한 경우 다시 정보 요청
  if (state == STATE_IDLE) {
    if (time_current - time_previous > QUERY_INTERVAL) {
      Serial.println("** Start to query current weather information...");

      state = STATE_RECEIVING_INFO;
      buffer = "";

      if (!sendATcommand(connectString, "OK", 10)) {  // TCP 연결 오류
        Serial.println("** Error in connecting to weather server.");
        Serial.println("   Try again in 10 minutes...");

        time_previous = millis();
        state = STATE_IDLE;
      }
      else {              // GET 메시지 전송
        sendATcommand(infoString, "OK\r\n>", 10);
        Serial.println("** Send GET message...");

        Serial1.println(dataString1);
        Serial1.println(dataString2);
        Serial1.println(dataString3);

        if (!sendATcommand("", "SEND OK", 5)) { // GET 메시지 전송 오류
          Serial.println("** Error in send GET message...");

          time_previous = millis();
          state = STATE_IDLE;
        }
        else {
          // GET 메시지를 정상적으로 전송하고 서버로부터 정보 수신 모드로 전환
          time_previous = millis();
          Serial.println("** GET message has been sent successfully...");
        }
      }
    }
  }
  // 서버로부터 정보 수신 대기
  else if (state == STATE_RECEIVING_INFO) {
    // 서버로부터 데이터 수신이 지연된 경우
    if (time_current - time_previous > SERVER_TIMEOUT) {
      Serial.println("** Timeout in receiving weather information.");
      Serial.println("   Try again in 10 minutes...");

      time_previous = millis();
      state = STATE_IDLE;
    }
    else {
      while (Serial1.available()) {
        char data = Serial1.read();
        buffer += data;
      }

      // 날씨 정보 수신이 끝났는지 검사
      if (buffer.indexOf("CLOSED") != -1) {
        Serial.println("** Received weather information...");

        processWeatherInformation();

        time_previous = millis();
        state = STATE_IDLE;
      }
    }
  }
}

void processWeatherInformation(void) {
  int index1, index2;
  String fieldInfo;
  char *key1 = "\"weather\"", *key2 = "\"main\"", *key3 = "\"temp\"";

  // weather.main 필드
  index1 = buffer.indexOf(key1);          // "weather" 검색
  index2 = buffer.indexOf(key2, index1);        // "main" 검색
  index1 = buffer.indexOf('\"', index2 + strlen(key2)); // 시작 큰따옴표 검색
  index2 = buffer.indexOf('\"', index1 + 1);            // 종료 큰따옴표 검색

  fieldInfo = buffer.substring(index1 + 1, index2);
  Serial.println();
  Serial.println("Current Weather in Busan     : " + fieldInfo);

  // main.temp 필드
  index1 = buffer.indexOf(key3, index2);        // "temp" 검색
  index2 = buffer.indexOf(':', index1 + strlen(key3));  // 시작 콜론 검색
  index1 = buffer.indexOf(',', index2 + 1);     // 종료 쉼표 검색

  fieldInfo = buffer.substring(index2 + 1, index1);
  float temperature = fieldInfo.toFloat() - 273.15;     // 섭씨 온도로 변환
  Serial.print("Current Temperature in Busan : ");
  Serial.println(temperature);
  Serial.println();
}
