#include <OneWire.h>

OneWire DS1820(2);      // 2번 핀에 온도 센서 연결

void setup(void) {
  Serial.begin(9600);
}

void loop(void) {
  byte data[9];       // 스크래치패드 데이터
  byte addr[8];       // 슬레이브 주소
  float celsius, fahrenheit;

  if (!DS1820.search(addr)) {   // 다음 1-와이어 슬레이브 검색
    Serial.println("No more addresses.");
    Serial.println();
    DS1820.reset_search();    // 슬레이브 검색 초기화
    delay(250);
    return;
  }

  Serial.print("ROM =");
  for (int i = 0; i < 8; i++) {   // 슬레이브 주소 출력
    Serial.write(' ');
    Serial.print(addr[i], HEX);
  }

  if (OneWire::crc8(addr, 7) != addr[7]) {  // 주소에 대한 CRC 검사
    Serial.println("CRC is not valid!");
    return;
  }

  Serial.println();
  if (addr[0] != 0x28) {      // 패밀리 코드검사
    Serial.println("The device found is not a DS18B20.");
    return;
  }

  DS1820.reset();       // 리셋
  DS1820.select(addr);      // 슬레이브 선택
  // 온도 변환 시작 명령으로 기생 전력 모드를 위해 power 매개변수를 1로 설정
  DS1820.write(0x44, 1);

  // 12비트 온도 변환을 위해서는 750ms 이상의 시간이 필요
  delay(1000);

  DS1820.reset();
  DS1820.select(addr);
  DS1820.write(0xBE);     // 스크래치 패드 데이터 읽기 명령

  Serial.print("  Data = ");
  for (int i = 0; i < 9; i++) {   // 9바이트 스크래치패드 데이터 읽기
    data[i] = DS1820.read();
    Serial.print(data[i], HEX);
    Serial.print(" ");
  }
  Serial.print(" CRC = ");
  // 스크래치패드 데이터에 대한 CRC 검사
  Serial.print(OneWire::crc8(data, 8), HEX);
  Serial.println();

  int16_t raw = (data[1] << 8) | data[0]; // 온도 레지스터 값
  byte cfg = (data[4] & 0x60);    // 설정 레지스터의 해상도 설정 값

  if (cfg == 0x00) raw = raw & ~7;  // 9 비트 해상도로 하위 3비트 제거
  else if (cfg == 0x20) raw = raw & ~3; // 10 비트 해상도로 하위 2비트 제거
  else if (cfg == 0x40) raw = raw & ~1; // 11 비트 해상도로 하위 1비트 제거

  celsius = (float)raw / 16.0;
  fahrenheit = celsius * 1.8 + 32.0;
  Serial.print("  Temperature = ");
  Serial.print(celsius);
  Serial.print(" Celsius, ");
  Serial.print(fahrenheit);
  Serial.println(" Fahrenheit");
}
