#include <Wire.h>
#include <RtcDS1307.h>
RtcDS1307<TwoWire> RTC(Wire);

void setup () {
  Serial.begin(115200);
  RTC.Begin();

  Serial.print("** Compile Time : ");
  RtcDateTime compiled = RtcDateTime(__DATE__, __TIME__);
  printDateTime(compiled);
  Serial.println();

  if (!RTC.GetIsRunning()) {    // 동작 상태 검사
    Serial.println("RTC was not running, starting now");
    RTC.SetIsRunning(true);   // RTC 시작
  }

  // 컴파일 시간보다 RTC 모듈의 시간이 오래된 경우 시간 업데이트
  RtcDateTime now = RTC.GetDateTime();
  if (now < compiled) {
    Serial.println("RTC is older than compile time! Updating date and time.");
    RTC.SetDateTime(compiled);
  }
}

void loop () {
  RtcDateTime now = RTC.GetDateTime();  // 시간 얻기
  printDateTime(now);     // 시간 출력
  delay(10000);       // 10초에 1번 출력
}

void printDateTime(const RtcDateTime dt) {
  Serial.printf("%04d/%02d/%02d, ", dt.Year(), dt.Month(), dt.Day());
  Serial.printf("%02d:%02d:%02d\n", dt.Hour(), dt.Minute(), dt.Second());
}
