#include <Wire.h>
#include <RtcDS1307.h>

const byte interruptPin = 13;    // 인터럽트 검사 (구형파 입력) 핀
volatile bool alarm = false;    // 인터럽트 발생 여부

RtcDS1307<TwoWire> RTC(Wire);

void setup() {
  Serial.begin(115200);

  pinMode(interruptPin, INPUT_PULLUP);
  // 하강 에지에서 ‘handleInterrupt’ 함수를 호출하도록 설정
  attachInterrupt(digitalPinToInterrupt(interruptPin), handleInterrupt, FALLING);

  RTC.Begin();        // RTC 모듈 시작

  // 1Hz의 구형파가 출력되도록 설정
  RTC.SetSquareWavePin(DS1307SquareWaveOut_1Hz);
}

void loop() {
  if (alarm == true) {      // 인터럽트 발생 여부 검사
    handleAlarm();
  }
}

void handleAlarm() {
  alarm = false;        // 인터럽트 처리 여부 표시

  // 인터럽트가 처리되는 시간 표시
  RtcDateTime timestamp = RTC.GetDateTime();
  Serial.print("Interrupt time : ");

  char time[10];
  sprintf(time, "%d:%d:%d",
          timestamp.Hour(), timestamp.Minute(), timestamp.Second());
  Serial.println(time);
}

void handleInterrupt() {      // 인터럽트 처리 함수
  alarm = true;       // 인터럽트 발생 표시
}
