#include <Wire.h>
#include <RtcDS3231.h>

const byte interruptPin = 13;    // 인터럽트 검사 핀
volatile bool alarm = false;    // 인터럽트 발생 여부

RtcDS3231<TwoWire> RTC(Wire);

void setup() {
  Serial.begin(115200);

  pinMode(interruptPin, INPUT_PULLUP);
  // 하강 에지에서 ‘handleInterrupt’ 함수를 호출하도록 설정
  attachInterrupt(digitalPinToInterrupt(interruptPin), handleInterrupt, FALLING);

  RTC.Begin();        // RTC 모듈 시작

  RTC.Enable32kHzPin(false);    // 32.768kHz 구형파 생성 금지
  // 1번 알람 사용 설정
  RTC.SetSquareWavePin(DS3231SquareWavePin_ModeAlarmOne);
  // 1번 알람의 알람 정보 설정
  DS3231AlarmOne alarm1(0, 0, 0, 0, DS3231AlarmOneControl_OncePerSecond);

  RTC.SetAlarmOne(alarm1);    // 알람 정보를 DS3231로 전송
  RTC.LatchAlarmsTriggeredFlags();  // 알람 발생 허용
}

void loop() {
  if (alarm == true) {      // 인터럽트 발생 여부 검사
    handleAlarm();
  }
}

void handleAlarm() {
  alarm = false;        // 인터럽트 처리 여부 표시

  // 인터럽트가 처리되는 시간 표시
  RtcDateTime timestamp = RTC.GetDateTime();
  Serial.print("Interrupt time : ");

  char time[10];
  sprintf(time, "%d:%d:%d",
          timestamp.Hour(), timestamp.Minute(), timestamp.Second());
  Serial.println(time);

  RTC.LatchAlarmsTriggeredFlags();  // 알람 발생 허용
}

void handleInterrupt() {      // 인터럽트 처리 함수
  alarm = true;       // 인터럽트 발생 표시
}
