#include <ESP8266WiFi.h>
#include <ESP8266HTTPClient.h>

char *ssid = "your_AP_name";
char *password = "password_to_your_AP";

void setup() {
  Serial.begin(115200);

  WiFi.mode(WIFI_STA);        // 스테이션 모드
  WiFi.disconnect();        // 기존 연결 종료

  Serial.print("Connecting to \'");
  Serial.print(ssid);
  Serial.println("\'");

  WiFi.begin(ssid, password);     // AP에 접속

  // AP에 접속이 될 때까지 대기
  while (WiFi.status() != WL_CONNECTED) {
    delay(500);
    Serial.print(".");
  }
  Serial.println();
  Serial.println("WiFi connected.");
}

void loop() {
  HTTPClient httpClient;        // HTTP 클라이언트

  String host = "http://api.openweathermap.org";
  String url = "/data/2.5/weather?id=1838524&APPID=";
  String key = "your_key_here";     // 32글자 길이

  httpClient.begin(host + url + key);
  int httpCode = httpClient.GET();      // 웹 서버 응답

  // 접속에 오류가 발생하면 음의 값을 반환함
  if (httpCode > 0) {
    Serial.printf("[HTTP] request from the client was handled : %d\n", httpCode);

    if (httpCode == HTTP_CODE_OK) {   // 웹 페이지 데이터 읽기
      String payload = httpClient.getString();
      processWeatherInformation(payload);
    }
  } else {
    Serial.printf("[HTTP] connection failed : %s\n",
                  httpClient.errorToString(httpCode).c_str());
  }

  httpClient.end();       // HTTP 클라이언트 중지

  delay(5000);
}

void processWeatherInformation(String buffer) {
  int index1, index2;
  String fieldInfo;
  char *key1 = "\"weather\"", *key2 = "\"main\"", *key3 = "\"temp\"";

  // weather.main 필드
  index1 = buffer.indexOf(key1);        // "weather" 검색
  index2 = buffer.indexOf(key2, index1);      // "main" 검색
  index1 = buffer.indexOf('\"', index2 + strlen(key2)); // 시작 큰따옴표 검색
  index2 = buffer.indexOf('\"', index1 + 1);    // 종료 큰따옴표 검색

  fieldInfo = buffer.substring(index1 + 1, index2);
  Serial.println();
  Serial.println("Current Weather in Busan     : " + fieldInfo);

  // main.temp 필드
  index1 = buffer.indexOf(key3, index2);      // "temp" 검색
  index2 = buffer.indexOf(':', index1 + strlen(key3));  // 시작 콜론 검색
  index1 = buffer.indexOf(',', index2 + 1);     // 종료 쉼표 검색

  fieldInfo = buffer.substring(index2 + 1, index1);
  float temperature = fieldInfo.toFloat() - 273.15;   // 섭씨 온도로 변환
  Serial.print("Current Temperature in Busan : ");
  Serial.println(temperature);
  Serial.println();
}
