#include <ESP8266WiFi.h>
#include <ESP8266HTTPClient.h>

#include "DHT.h"
#define DHTPIN    2   // DHT22 센서 연결 핀
#define DHTTYPE DHT22   // 센서의 종류

char *ssid = "your_AP_name";
char *password = "password_to_your_AP";

DHT dht(DHTPIN, DHTTYPE);   // 온습도 측정 전역 객체 생성

void setup() {
  Serial.begin(115200);

  dht.begin();        // 센서 초기화

  WiFi.mode(WIFI_STA);      // 스테이션 모드
  WiFi.disconnect();      // 기존 연결 종료

  Serial.print("Connecting to \'");
  Serial.print(ssid);
  Serial.println("\'");

  WiFi.begin(ssid, password);   // AP에 접속

  // AP에 접속이 될 때까지 대기
  while (WiFi.status() != WL_CONNECTED) {
    delay(500);
    Serial.print(".");
  }
  Serial.println();
  Serial.println("WiFi connected.");
}

void loop() {
  float h = dht.readHumidity();   // 습도 읽기
  float t = dht.readTemperature();  // 온도 읽기

  if (isnan(h) || isnan(t)) {   // 센서값 검사
    Serial.println("** Error in reading DHT sensor!");
    return;
  }

  Serial.print("Humidity    : ");
  Serial.print(h);
  Serial.println(" %");
  Serial.print("Temperature : ");
  Serial.print(t);
  Serial.println(" C");
  Serial.println();

  HTTPClient httpClient;      // HTTP 클라이언트

  String host = "http://api.thingspeak.com";
  String url = "/update?key=";
  String key = "your_write_key_here"; // 32글자 길이
  String field1 = "&field1=";
  String field2 = "&field2=";

  httpClient.begin(host + url + key + field1 + t + field2 + h);
  int httpCode = httpClient.GET();    // 웹 서버 응답

  // 접속에 오류가 발생하면 음의 값을 반환함
  if (httpCode > 0) {
    Serial.printf("[HTTP] Temperature & Humidity data was logged...\n");
  } else {
    Serial.printf("[HTTP] Connection failed : %s\n",
                  httpClient.errorToString(httpCode).c_str());
  }

  httpClient.end();     // HTTP 클라이언트 중지

  delay(20000);
}
