#include <ESP8266WiFi.h>
#include <ESP8266WebServer.h>

char *ssid = "your_AP_name";
char *password = "password_to_your_AP";

ESP8266WebServer server(80);    // 웹 서버 객체

void handleRoot() {
  server.send(200, "text/plain", "Hello from esp8266!");
}

void handleTest() {
  server.send(200, "text/plain", "Test directory");
}

void handleNotFound() {     // 존재하지 않는 주소로 접속
  String message = "File Not Found\n\n";
  message += "URI: ";
  message += server.uri();
  message += "\nArguments: ";
  message += server.args();
  message += "\n";
  for (uint8_t i = 0; i < server.args(); i++) {
    message += " " + server.argName(i) + ": " + server.arg(i) + "\n";
  }
  server.send(404, "text/plain", message);
}

void setup(void) {
  Serial.begin(115200);

  Serial.print("Connecting to \'");
  Serial.print(ssid);
  Serial.println("\'");

  WiFi.begin(ssid, password);   // AP에 접속

  // AP에 접속이 될 때까지 대기
  while (WiFi.status() != WL_CONNECTED) {
    delay(500);
    Serial.print(".");
  }
  Serial.println();
  Serial.println("WiFi connected.");

  Serial.print("Server IP address: ");
  Serial.println(WiFi.localIP());   // 서버에 할당된 주소

  // 접속 주소에 따른 처리 함수 등록
  server.on("/", handleRoot);
  server.on("/test", handleTest);
  server.onNotFound(handleNotFound);

  server.begin();       // 웹 서버 시작
  Serial.println("HTTP server started");
}

void loop(void) {
  server.handleClient();      // 클라이언트 요청 처리
}
