#include <ESP8266WiFi.h>
#include <ESP8266WebServer.h>
#include "DHT.h"
#define DHTPIN    2   // DHT22 센서 연결 핀
#define DHTTYPE DHT22   // 센서의 종류

DHT dht(DHTPIN, DHTTYPE);   // 전역 객체 생성

char *ssid = "your_AP_name";
char *password = "password_to_your_AP";

ESP8266WebServer server(80);    // 웹 서버 객체

void handleRoot() {
  float h = dht.readHumidity();   // 습도
  float t = dht.readTemperature();  // 온도

  server.sendHeader("Refresh", "10"); // 10초 마다 자동 갱신

  if (isnan(h) || isnan(t)) {
    server.send(200, "text/plain", "Error in reading sensor.");
  }

  String message = "";
  message += "Temerature : ";
  message += t;
  message += ", Humidity : ";
  message += h;

  server.send(200, "text/plain", message);
}

void setup(void) {
  Serial.begin(115200);

  dht.begin();        // 센서 초기화

  Serial.print("Connecting to \'");
  Serial.print(ssid);
  Serial.println("\'");

  WiFi.begin(ssid, password);   // AP에 접속

  // AP에 접속이 될 때까지 대기
  while (WiFi.status() != WL_CONNECTED) {
    delay(500);
    Serial.print(".");
  }
  Serial.println();
  Serial.println("WiFi connected.");

  Serial.print("Server IP address: ");
  Serial.println(WiFi.localIP());   // 서버에 할당된 주소

  server.on("/", handleRoot);   // 처리 함수 등록

  server.begin();       // 웹 서버 시작
  Serial.println("HTTP server started");
}

void loop(void) {
  server.handleClient();      // 클라이언트 요청 처리
}
