#include <ESP8266WiFi.h>
#include <ESP8266WebServer.h>

char *ssid = "your_AP_name";
char *password = "password_to_your_AP";

int LED_pin = 4;        // LED 연결 핀
int brightness = 0;     // LED는 꺼진 상태로 시작

ESP8266WebServer server(80);    // 웹 서버 객체

void setup(void){
  Serial.begin(115200);

  Serial.print("Connecting to \'");
  Serial.print(ssid);
  Serial.println("\'");

  WiFi.begin(ssid, password);   // AP에 접속

  // AP에 접속이 될 때까지 대기
  while (WiFi.status() != WL_CONNECTED) {
    delay(500);
    Serial.print(".");
  }
  Serial.println();
  Serial.println("WiFi connected.");

  Serial.print("Server IP address: ");
  Serial.println(WiFi.localIP());   // 서버에 할당된 주소

  // 처리 함수 등록
  server.on("/", handleRoot);
  server.on("/led.cgi", handleBrightness);

  server.begin();       // 웹 서버 시작
  Serial.println("HTTP server started");

  pinMode(LED_pin, OUTPUT);
}

void showMainPage() {
  String message = "";
  message += "<html>";
  message += "<body>";

  message += "LED brightness (0~1023) is "; // 현태 LED 밝기
  message += brightness;

  // CGI를 통해 사용자 입력을 받음
  message += "<br />";
  message += "<FORM method=\"get\" action=\"/led.cgi\">";
  message += "<P>Brightness <INPUT type=\"text\" name=\"brightness\">";
  message += "<P><INPUT type=\"submit\" value=\"Submit\"> </FORM>";

  message += "</body>";
  message += "</html>";

  server.send(200, "text/html", message);   // HTML 형식 페이지 전송
}

void handleBrightness() {      // LED 밝기 조절 페이지
  if(server.argName(0) == "brightness"){  // 인자 이름
    int state = server.arg(0).toInt();  // 인자의 값

    brightness = state;
    analogWrite(LED_pin, brightness); // LED 상태 반영

    showMainPage();
  }
}

void handleRoot() {
  showMainPage();
}

void loop(void){
  server.handleClient();      // 클라이언트 요청 처리
}
